@@ -17,10 +17,10 @@ import enum
 import json
 import os
 import re
+import urllib
 
 from oslo_config import cfg
 from oslo_log import log as logging
-from six.moves import urllib
 import webob
 
 from cinder.api import api_utils
@@ -49,7 +49,8 @@ LOG = logging.getLogger(__name__)
 _FILTERS_COLLECTION = None
 
 ATTRIBUTE_CONVERTERS = {'name~': 'display_name~',
-                        'description~': 'display_description~'}
+                        'description~': 'display_description~',
+                        'consumes_quota': 'use_quota'}
 
 
 METADATA_TYPES = enum.Enum('METADATA_TYPES', 'user image')
@@ -446,7 +447,11 @@ def reject_invalid_filters(context, filters, resource,
                 invalid_filters.append(key)
     if invalid_filters:
         if 'all_tenants' in invalid_filters:
+            # NOTE: this is a special case: the cinderclient always adds
+            # 'all_tenants', so we don't want to hold that against a non-admin
+            # user and we silently ignore it.  See Bug #1917574.
             invalid_filters.remove('all_tenants')
+            filters.pop('all_tenants')
         if len(invalid_filters) == 0:
             return
         raise webob.exc.HTTPBadRequest(
