@@ -15,18 +15,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""
-Handles all requests relating to the volume backups service.
-"""
+"""Handles all requests relating to the volume backups service."""
 
 from datetime import datetime
+import random
+
 from eventlet import greenthread
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import strutils
 from pytz import timezone
-import random
 
 from cinder.backup import rpcapi as backup_rpcapi
 from cinder.common import constants
@@ -41,17 +40,17 @@ from cinder.policies import backups as policy
 import cinder.policy
 from cinder import quota
 from cinder import quota_utils
+from cinder.scheduler import rpcapi as scheduler_rpcapi
 import cinder.volume
-from cinder.volume import utils as volume_utils
 
-backup_api_opts = [
+backup_opts = [
     cfg.BoolOpt('backup_use_same_host',
                 default=False,
                 help='Backup services use same backend.')
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(backup_api_opts)
+CONF.register_opts(backup_opts)
 LOG = logging.getLogger(__name__)
 QUOTAS = quota.QUOTAS
 IMPORT_VOLUME_ID = '00000000-0000-0000-0000-000000000000'
@@ -60,10 +59,11 @@ IMPORT_VOLUME_ID = '00000000-0000-0000-0000-000000000000'
 class API(base.Base):
     """API for interacting with the volume backup manager."""
 
-    def __init__(self, db=None):
+    def __init__(self):
         self.backup_rpcapi = backup_rpcapi.BackupAPI()
+        self.scheduler_rpcapi = scheduler_rpcapi.SchedulerAPI()
         self.volume_api = cinder.volume.API()
-        super(API, self).__init__(db)
+        super().__init__()
 
     def get(self, context, backup_id):
         backup = objects.Backup.get_by_id(context, backup_id)
@@ -228,10 +228,6 @@ class API(base.Base):
             raise exception.InvalidVolume(reason=msg)
 
         previous_status = volume['status']
-        volume_host = volume_utils.extract_host(volume.host, 'host')
-        availability_zone = availability_zone or volume.availability_zone
-        host = self._get_available_backup_service_host(volume_host,
-                                                       availability_zone)
 
         # Reserve a quota before setting volume status and backup status
         try:
@@ -247,8 +243,9 @@ class API(base.Base):
         # incremental backup.
         latest_backup = None
         if incremental:
-            backups = objects.BackupList.get_all_by_volume(context.elevated(),
-                                                           volume_id)
+            backups = objects.BackupList.get_all_by_volume(
+                context, volume_id, volume['project_id'],
+                filters={'project_id': context.project_id})
             if backups.objects:
                 # NOTE(xyang): The 'data_timestamp' field records the time
                 # when the data on the volume was first saved. If it is
@@ -274,13 +271,18 @@ class API(base.Base):
                                          < snapshot['created_at']))
                     else datetime(1, 1, 1, 1, 1, 1, tzinfo=timezone('UTC')))
             else:
+                QUOTAS.rollback(context, reservations)
                 msg = _('No backups available to do an incremental backup.')
                 raise exception.InvalidBackup(reason=msg)
 
         parent_id = None
+        parent = None
+
         if latest_backup:
+            parent = latest_backup
             parent_id = latest_backup.id
             if latest_backup['status'] != fields.BackupStatus.AVAILABLE:
+                QUOTAS.rollback(context, reservations)
                 msg = _('The parent backup must be available for '
                         'incremental backup.')
                 raise exception.InvalidBackup(reason=msg)
@@ -309,10 +311,9 @@ class API(base.Base):
                 'container': container,
                 'parent_id': parent_id,
                 'size': volume['size'],
-                'host': host,
-                'availability_zone': availability_zone,
                 'snapshot_id': snapshot_id,
                 'data_timestamp': data_timestamp,
+                'parent': parent,
                 'metadata': metadata or {}
             }
             backup = objects.Backup(context=context, **kwargs)
@@ -329,10 +330,7 @@ class API(base.Base):
                 finally:
                     QUOTAS.rollback(context, reservations)
 
-        # TODO(DuncanT): In future, when we have a generic local attach,
-        #                this can go via the scheduler, which enables
-        #                better load balancing and isolation of services
-        self.backup_rpcapi.create_backup(context, backup)
+        self.scheduler_rpcapi.create_backup(context, backup)
 
         return backup
 
@@ -501,6 +499,23 @@ class API(base.Base):
             msg = _('Provided backup record is missing size attribute')
             raise exception.InvalidInput(reason=msg)
 
+        # Try to get the backup with that ID in all projects even among
+        # deleted entries (we reuse soft-deleted backups).
+        try:
+            backup = objects.BackupImport.get_by_id(
+                context.elevated(read_deleted='yes'),
+                backup_record['id'],
+                project_only=False)
+
+            # If record exists and it's not deleted we cannot proceed
+            # with the import
+            if backup.status != fields.BackupStatus.DELETED:
+                msg = _('Backup already exists in database.')
+                raise exception.InvalidBackup(reason=msg)
+        except exception.BackupNotFound:
+            pass
+
+        # Check that we're under limit by reserving quota
         try:
             reserve_opts = {'backups': 1,
                             'backup_gigabytes': backup_record['size']}
@@ -522,30 +537,16 @@ class API(base.Base):
         }
 
         try:
-            try:
-                # Try to get the backup with that ID in all projects even among
-                # deleted entries.
-                backup = objects.BackupImport.get_by_id(
-                    context.elevated(read_deleted='yes'),
-                    backup_record['id'],
-                    project_only=False)
-
-                # If record exists and it's not deleted we cannot proceed
-                # with the import
-                if backup.status != fields.BackupStatus.DELETED:
-                    msg = _('Backup already exists in database.')
-                    raise exception.InvalidBackup(reason=msg)
-
-                # Otherwise we'll "revive" delete backup record
+            if backup:
+                # "revive" the soft-deleted backup record retrieved earlier
                 backup.update(kwargs)
                 backup.save()
-                QUOTAS.commit(context, reservations)
-            except exception.BackupNotFound:
-                # If record doesn't exist create it with the specific ID
+            else:
+                # create a new backup with the specified ID
                 backup = objects.BackupImport(context=context,
                                               id=backup_record['id'], **kwargs)
                 backup.create()
-                QUOTAS.commit(context, reservations)
+            QUOTAS.commit(context, reservations)
         except Exception:
             with excutils.save_and_reraise_exception():
                 try:
