@@ -47,13 +47,6 @@ core_opts = [
 CONF.register_cli_opts(core_opts)
 
 api_opts = [
-    cfg.BoolOpt('enable_v2_api',
-                default=True,
-                deprecated_for_removal=True,
-                help="DEPRECATED: Deploy v2 of the Cinder API."),
-    cfg.BoolOpt('enable_v3_api',
-                default=True,
-                help="Deploy v3 of the Cinder API."),
     cfg.BoolOpt('api_rate_limit',
                 default=True,
                 help='Enables or disables rate limit of the API.'),
@@ -191,6 +184,12 @@ image_opts = [
     cfg.StrOpt('glance_ca_certificates_file',
                help='Location of ca certificates file to use for glance '
                     'client requests.'),
+    cfg.StrOpt('glance_certfile',
+               help='Location of certificate file to use for glance '
+                    'client requests.'),
+    cfg.StrOpt('glance_keyfile',
+               help='Location of certificate key file to use for glance '
+                    'client requests.'),
     cfg.IntOpt('glance_request_timeout',
                help='http/https timeout value for glance operations. If no '
                     'value (None) is supplied here, the glanceclient default '
