@@ -12,6 +12,7 @@
 #    under the License.
 
 import datetime
+from http import HTTPStatus
 import json
 from unittest import mock
 
@@ -21,7 +22,6 @@ import iso8601
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
-from six.moves import http_client
 import webob
 
 from cinder.api import api_utils
@@ -31,6 +31,7 @@ from cinder.api import microversions as mv
 from cinder.api.v2.views.volumes import ViewBuilder
 from cinder.api.v3 import volumes
 from cinder.backup import api as backup_api
+from cinder.common import constants as cinder_constants
 from cinder import context
 from cinder import db
 from cinder import exception
@@ -50,6 +51,29 @@ from cinder.volume import api as vol_get
 
 DEFAULT_AZ = "zone1:host1"
 
+# DDT data for testing whether an 'encryption_key_id' should appear in a
+# volume's or backup's details (also used by test_backups.py).
+ENCRYPTION_KEY_ID_IN_DETAILS = {
+    'expected_in_details': True,
+    'encryption_key_id': fake.ENCRYPTION_KEY_ID,
+    'version': mv.ENCRYPTION_KEY_ID_IN_DETAILS,
+}, {
+    # No encryption ID to display
+    'expected_in_details': False,
+    'encryption_key_id': None,
+    'version': mv.ENCRYPTION_KEY_ID_IN_DETAILS,
+}, {
+    # Fixed key ID should not be displayed
+    'expected_in_details': False,
+    'encryption_key_id': cinder_constants.FIXED_KEY_ID,
+    'version': mv.ENCRYPTION_KEY_ID_IN_DETAILS,
+}, {
+    # Unsupported microversion
+    'expected_in_details': False,
+    'encryption_key_id': fake.ENCRYPTION_KEY_ID,
+    'version': mv.get_prior_version(mv.ENCRYPTION_KEY_ID_IN_DETAILS),
+}
+
 
 @ddt.ddt
 class VolumeApiTest(test.TestCase):
@@ -319,6 +343,45 @@ class VolumeApiTest(test.TestCase):
         volumes = res_dict['volumes']
         self.assertEqual(2, len(volumes))
 
+    @ddt.data(('true', 0), ('false', 1))
+    @ddt.unpack
+    def test_volume_list_with_quota_filter(self, use_quota, expected_index):
+        volumes = (test_utils.create_volume(self.ctxt, host='test_host1',
+                                            cluster_name='cluster1',
+                                            volume_type_id=None,
+                                            use_quota=True,
+                                            availability_zone='nova1'),
+                   test_utils.create_volume(self.ctxt, host='test_host2',
+                                            cluster_name='cluster2',
+                                            volume_type_id=None,
+                                            use_quota=False,
+                                            availability_zone='nova2'))
+        req = fakes.HTTPRequest.blank(
+            '/v3/volumes?consumes_quota=%s' % use_quota, version=mv.USE_QUOTA)
+        res_dict = self.controller.detail(req)
+        self.assertEqual(1, len(res_dict['volumes']))
+        self.assertEqual(volumes[expected_index].id,
+                         res_dict['volumes'][0]['id'])
+
+    def test_volume_list_without_quota_filter(self):
+        num_vols = 4
+        vol_ids = set()
+        # Half of the volumes will use quota, the other half won't
+        for i in range(num_vols):
+            vol = test_utils.create_volume(self.ctxt,
+                                           use_quota=bool(i % 2),
+                                           host='test_host',
+                                           cluster_name='cluster',
+                                           volume_type_id=None,
+                                           availability_zone='nova1')
+            vol_ids.add(vol.id)
+        req = fakes.HTTPRequest.blank('/v3/volumes', version=mv.USE_QUOTA)
+        res_dict = self.controller.detail(req)
+
+        res_vol_ids = {v['id'] for v in res_dict['volumes']}
+        self.assertEqual(num_vols, len(res_vol_ids))
+        self.assertEqual(vol_ids, res_vol_ids)
+
     def _fake_volumes_summary_request(self,
                                       version=mv.VOLUME_SUMMARY,
                                       all_tenant=False,
@@ -556,7 +619,6 @@ class VolumeApiTest(test.TestCase):
             'consistencygroup': None,
             'availability_zone': availability_zone,
             'scheduler_hints': None,
-            'multiattach': False,
             'group': test_group,
         }
 
@@ -748,7 +810,7 @@ class VolumeApiTest(test.TestCase):
         res = req.get_response(fakes.wsgi_app(
             fake_auth_context=self.ctxt))
         res_dict = jsonutils.loads(res.body)
-        self.assertEqual(http_client.ACCEPTED, res.status_int)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
         self.assertIn('id', res_dict['volume'])
 
     @ddt.data('fake_host', '', 1234, '          ')
@@ -856,6 +918,47 @@ class VolumeApiTest(test.TestCase):
         else:
             self.assertNotIn('provider_id', res_dict['volume'])
 
+    @ddt.data(*ENCRYPTION_KEY_ID_IN_DETAILS)
+    @ddt.unpack
+    def test_volume_show_with_encryption_key_id(self,
+                                                expected_in_details,
+                                                encryption_key_id,
+                                                version):
+        volume = test_utils.create_volume(self.ctxt,
+                                          testcase_instance=self,
+                                          volume_type_id=None,
+                                          encryption_key_id=encryption_key_id)
+
+        req = fakes.HTTPRequest.blank('/v3/volumes/%s' % volume.id,
+                                      version=version)
+        volume_details = self.controller.show(req, volume.id)['volume']
+
+        if expected_in_details:
+            self.assertIn('encryption_key_id', volume_details)
+        else:
+            self.assertNotIn('encryption_key_id', volume_details)
+
+    @ddt.data(
+        (True, True, mv.USE_QUOTA),
+        (True, False, mv.USE_QUOTA),
+        (False, True, mv.get_prior_version(mv.USE_QUOTA)),
+        (False, False, mv.get_prior_version(mv.USE_QUOTA)),
+    )
+    @ddt.unpack
+    def test_volume_show_with_use_quota(self, present, value, microversion):
+        volume = test_utils.create_volume(self.ctxt,
+                                          volume_type_id=None,
+                                          use_quota=value)
+
+        req = fakes.HTTPRequest.blank('/v3/volumes/%s' % volume.id,
+                                      version=microversion)
+        volume_details = self.controller.show(req, volume.id)['volume']
+
+        if present:
+            self.assertIs(value, volume_details['consumes_quota'])
+        else:
+            self.assertNotIn('consumes_quota', volume_details)
+
     def _fake_create_volume(self, size=1):
         vol = {
             'display_name': 'fake_volume1',
@@ -928,12 +1031,15 @@ class VolumeApiTest(test.TestCase):
         req.headers = mv.get_mv_header(mv.VOLUME_REVERT)
         req.api_version_request = mv.get_api_version(
             mv.VOLUME_REVERT)
+        req.environ['cinder.context'] = self.ctxt
         # update volume's status failed
         mock_update.side_effect = [False, True]
 
-        self.assertRaises(webob.exc.HTTPConflict, self.controller.revert,
-                          req, fake_volume['id'], {'revert': {'snapshot_id':
-                                                   fake_snapshot['id']}})
+        self.assertRaises(webob.exc.HTTPConflict,
+                          self.controller.revert,
+                          req,
+                          fake_volume['id'],
+                          {'revert': {'snapshot_id': fake_snapshot['id']}})
 
         # update snapshot's status failed
         mock_update.side_effect = [True, False]
@@ -1082,3 +1188,25 @@ class VolumeApiTest(test.TestCase):
         volumes = res_dict['volumes']
         self.assertEqual(1, len(volumes))
         self.assertEqual(vols[1].id, volumes[0]['id'])
+
+    def test_create_volume_with_multiattach_param(self):
+        """Tests creating a volume with multiattach=True but no multiattach
+
+        volume type.
+
+        This test verifies that providing the multiattach parameter will error
+        out the request since it is removed and the recommended way is to
+        create a multiattach volume using a multiattach volume type.
+        """
+        req = fakes.HTTPRequest.blank('/v3/volumes')
+
+        body = {'volume': {
+                'name': 'test name',
+                'description': 'test desc',
+                'size': 1,
+                'multiattach': True}}
+        exc = self.assertRaises(webob.exc.HTTPBadRequest,
+                                self.controller.create,
+                                req, body=body)
+        self.assertIn("multiattach parameter has been removed",
+                      exc.explanation)
