@@ -16,16 +16,22 @@
 
 import datetime
 import itertools
+import traceback
+from unittest import mock
 
+import ddt
 import glanceclient.exc
-import mock
+from keystoneauth1.loading import session as ks_session
+from keystoneauth1 import session
 from oslo_config import cfg
+import six
 
 from cinder import context
 from cinder import exception
 from cinder.image import glance
-from cinder import test
-from cinder.tests.unit.glance import stubs as glance_stubs
+from cinder import service_auth
+from cinder.tests.unit.image import glance_stubs
+from cinder.tests.unit import test
 
 
 CONF = cfg.CONF
@@ -71,6 +77,7 @@ class TestGlanceSerializer(test.TestCase):
         self.assertEqual(metadata, glance._convert_from_string(converted))
 
 
+@ddt.ddt
 class TestGlanceImageService(test.TestCase):
     """Tests the Glance image service.
 
@@ -103,10 +110,10 @@ class TestGlanceImageService(test.TestCase):
         self.service = self._create_image_service(client)
         self.context = context.RequestContext('fake', 'fake', auth_token=True)
         self.context.service_catalog = service_catalog
-        self.mock_object(glance.time, 'sleep', mock.Mock(return_value=None))
+        self.mock_object(glance.time, 'sleep', return_value=None)
 
     def _create_image_service(self, client):
-        def _fake_create_glance_client(context, netloc, use_ssl, version):
+        def _fake_create_glance_client(context, netloc, use_ssl):
             return client
 
         self.mock_object(glance, '_create_glance_client',
@@ -125,11 +132,46 @@ class TestGlanceImageService(test.TestCase):
         fixture.update(kwargs)
         return fixture
 
+    @staticmethod
+    def _make_image_member_fixtures(**kwargs):
+        fixtures = []
+        fixture = {'status': None,
+                   'image_id': None,
+                   'member_id': None,
+                   'created_at': '2018-03-14T21:48:13Z',
+                   'updated_at': '2018-03-14T21:50:51Z',
+                   'schema': '/v2/schemas/member'}
+        fixture.update(kwargs)
+        fixtures.append(fixture)
+        return fixtures
+
     def _make_datetime_fixture(self):
         return self._make_fixture(created_at=self.NOW_GLANCE_FORMAT,
                                   updated_at=self.NOW_GLANCE_FORMAT,
                                   deleted_at=self.NOW_GLANCE_FORMAT)
 
+    def test_list_members(self):
+        fixture = {'status': None,
+                   'image_id': None,
+                   'member_id': None,
+                   'created_at': '2018-03-14T21:48:13Z',
+                   'updated_at': '2018-03-14T21:50:51Z',
+                   'schema': '/v2/schemas/member'}
+        image_id = '97c1ef11-3a64-4756-9f8c-7f9fb5abe09f'
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        fixture['status'] = 'accepted'
+        fixture['image_id'] = image_id
+        fixture['member_id'] = member_id
+        with mock.patch.object(self.service, '_client') as client_mock:
+            client_mock.call.return_value = self._make_image_member_fixtures(
+                image_id=image_id, member_id=member_id, status='accepted')
+            result = self.service.list_members(self.context, image_id)
+        self.assertEqual([fixture], result)
+        client_mock.call.assert_called_once_with(self.context,
+                                                 'list',
+                                                 controller='image_members',
+                                                 image_id=image_id)
+
     def test_get_api_servers(self):
         result = glance.get_api_servers(self.context)
         expected = (u'example.com:9292', False)
@@ -157,7 +199,6 @@ class TestGlanceImageService(test.TestCase):
         expected = {
             'id': image_id,
             'name': 'test image',
-            'is_public': False,
             'protected': False,
             'size': None,
             'min_disk': None,
@@ -167,16 +208,17 @@ class TestGlanceImageService(test.TestCase):
             'checksum': None,
             'created_at': self.NOW_DATETIME,
             'updated_at': self.NOW_DATETIME,
-            'deleted_at': None,
             'deleted': None,
             'status': None,
-            'properties': {'instance_id': '42', 'user_id': 'fake'},
+            'properties': {'instance_id': '42', 'is_public': False,
+                           'user_id': 'fake'},
             'owner': None,
+            'visibility': None,
         }
-        self.assertDictMatch(expected, image_meta)
+        self.assertDictEqual(expected, image_meta)
 
         image_metas = self.service.detail(self.context)
-        self.assertDictMatch(expected, image_metas[0])
+        self.assertDictEqual(expected, image_metas[0])
 
     def test_create_without_instance_id(self):
         """Test Creating images without instance_id.
@@ -192,7 +234,6 @@ class TestGlanceImageService(test.TestCase):
         expected = {
             'id': image_id,
             'name': 'test image',
-            'is_public': False,
             'protected': False,
             'size': None,
             'min_disk': None,
@@ -202,14 +243,114 @@ class TestGlanceImageService(test.TestCase):
             'checksum': None,
             'created_at': self.NOW_DATETIME,
             'updated_at': self.NOW_DATETIME,
-            'deleted_at': None,
             'deleted': None,
             'status': None,
-            'properties': {},
+            'properties': {'is_public': False},
             'owner': None,
+            'visibility': None,
         }
         actual = self.service.show(self.context, image_id)
-        self.assertDictMatch(expected, actual)
+        self.assertDictEqual(expected, actual)
+
+    def test_show_shared_image_membership_success(self):
+        """Test Create Shared Image Membership Success
+
+        Ensure we can get access to a shared image
+        """
+        fixture = {'name': 'test image', 'is_public': False,
+                   'protected': False, 'visibility': 'shared'}
+        # pid = self.context.project_id
+        image_id = self.service.create(self.context, fixture)['id']
+        image = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'is_public': False},
+            'owner': None,
+            'visibility': None,
+        }
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        with mock.patch.object(self.service, '_client') as client_mock:
+            with mock.patch.object(
+                    self.service, '_translate_from_glance') as tg_mock:
+                tg_mock.return_value = {}
+                mock_image = mock.Mock()
+                mock_image.is_public = False
+                mock_image.properties = {'is_public': False}
+                mock_image.visibility = 'shared'
+                mock_image.keys.return_value = image.keys()
+                client_mock.call.side_effect = [
+                    mock_image,
+                    self._make_image_member_fixtures(image_id=image_id,
+                                                     member_id=member_id,
+                                                     status='accepted')]
+                self.context.project_id = member_id
+                self.context.is_admin = False
+                self.context.user_id = image_id
+                self.context.auth_token = False
+                self.service.show(self.context, image_id)
+
+    def test_show_shared_image_membership_fail_status(self):
+        """Test Create Shared Image Membership Failure
+
+        Ensure we can't get access to a shared image with the wrong membership
+        status (in this case 'pending')
+        """
+        fixture = {'name': 'test image', 'is_public': False,
+                   'protected': False, 'visibility': 'shared'}
+        # pid = self.context.project_id
+        image_id = self.service.create(self.context, fixture)['id']
+        image = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'is_public': False},
+            'owner': None,
+            'visibility': None,
+        }
+        member_id = '50fcc79f25524744a2c34682a1a74914'
+        with mock.patch.object(self.service, '_client') as client_mock:
+            with mock.patch.object(
+                    self.service, '_translate_from_glance') as tg_mock:
+                tg_mock.return_value = {}
+                mock_image = mock.Mock()
+                mock_image.is_public = False
+                mock_image.properties = {'is_public': False}
+                mock_image.visibility = 'shared'
+                mock_image.keys.return_value = image.keys()
+                client_mock.call.side_effect = [
+                    mock_image,
+                    self._make_image_member_fixtures(image_id=image_id,
+                                                     member_id=member_id,
+                                                     status='pending')]
+                self.context.project_id = member_id
+                self.context.is_admin = False
+                self.context.user_id = image_id
+                self.context.auth_token = False
+                self.assertRaises(exception.ImageNotFound,
+                                  self.service.show,
+                                  self.context,
+                                  image_id)
 
     def test_create(self):
         fixture = self._make_fixture(name='test image')
@@ -248,21 +389,10 @@ class TestGlanceImageService(test.TestCase):
 
         self.assertEqual(1, len(image_metas))
         self.assertEqual('test image', image_metas[0]['name'])
-        self.assertFalse(image_metas[0]['is_public'])
-
-    def test_detail_v1(self):
-        """Confirm we send is_public = None as default when using Glance v1."""
-        self.override_config('glance_api_version', 1)
-        with mock.patch.object(self.service, '_client') as client_mock:
-            client_mock.return_value = []
-            result = self.service.detail(self.context)
-        self.assertListEqual([], result)
-        client_mock.call.assert_called_once_with(self.context, 'list',
-                                                 filters={'is_public': 'none'})
+        self.assertEqual('private', image_metas[0]['visibility'])
 
     def test_detail_v2(self):
         """Check we don't send is_public key by default with Glance v2."""
-        self.override_config('glance_api_version', 2)
         with mock.patch.object(self.service, '_client') as client_mock:
             client_mock.return_value = []
             result = self.service.detail(self.context)
@@ -284,10 +414,9 @@ class TestGlanceImageService(test.TestCase):
             expected = {
                 'id': ids[i],
                 'status': None,
-                'is_public': None,
                 'protected': None,
                 'name': 'TestImage %d' % (i),
-                'properties': {},
+                'properties': {'properties': {}},
                 'size': None,
                 'min_disk': None,
                 'min_ram': None,
@@ -296,12 +425,12 @@ class TestGlanceImageService(test.TestCase):
                 'checksum': None,
                 'created_at': self.NOW_DATETIME,
                 'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
                 'deleted': None,
                 'owner': None,
+                'visibility': None,
             }
 
-            self.assertDictMatch(expected, meta)
+            self.assertDictEqual(expected, meta)
             i = i + 1
 
     def test_detail_limit(self):
@@ -342,10 +471,9 @@ class TestGlanceImageService(test.TestCase):
             expected = {
                 'id': ids[i],
                 'status': None,
-                'is_public': None,
                 'protected': None,
                 'name': 'TestImage %d' % (i),
-                'properties': {},
+                'properties': {'properties': {}},
                 'size': None,
                 'min_disk': None,
                 'min_ram': None,
@@ -354,11 +482,11 @@ class TestGlanceImageService(test.TestCase):
                 'checksum': None,
                 'created_at': self.NOW_DATETIME,
                 'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
                 'deleted': None,
                 'owner': None,
+                'visibility': None,
             }
-            self.assertDictMatch(expected, meta)
+            self.assertDictEqual(expected, meta)
             i = i + 1
 
     def test_detail_invalid_marker(self):
@@ -382,23 +510,63 @@ class TestGlanceImageService(test.TestCase):
         new_image_data = self.service.show(self.context, image_id)
         self.assertEqual('new image name', new_image_data['name'])
 
-    def test_update_v2(self):
-        self.flags(glance_api_version=2)
-        self.test_update()
-
     def test_update_with_data(self):
         fixture = self._make_fixture(name='test image')
         image = self.service.create(self.context, fixture)
         image_id = image['id']
+        fixture['name'] = 'new image name'
         data = '*' * 256
         self.service.update(self.context, image_id, fixture, data=data)
 
         new_image_data = self.service.show(self.context, image_id)
         self.assertEqual(256, new_image_data['size'])
+        self.assertEqual('new image name', new_image_data['name'])
 
-    def test_update_with_data_v2(self):
-        self.flags(glance_api_version=2)
-        self.test_update_with_data()
+    @mock.patch.object(glance.GlanceImageService, '_translate_from_glance')
+    @mock.patch.object(glance.GlanceImageService, 'show')
+    def test_update_purge_props(self, show, translate_from_glance):
+        image_id = mock.sentinel.image_id
+        client = mock.Mock(call=mock.Mock())
+        service = glance.GlanceImageService(client=client)
+
+        image_meta = {'properties': {'k1': 'v1'}}
+        show.return_value = {'properties': {'k2': 'v2'}}
+        translate_from_glance.return_value = image_meta.copy()
+
+        ret = service.update(self.context, image_id, image_meta)
+        self.assertDictEqual(image_meta, ret)
+        client.call.assert_called_once_with(
+            self.context, 'update', image_id, k1='v1', remove_props=['k2'])
+
+    @mock.patch.object(glance.GlanceImageService, '_translate_from_glance')
+    @mock.patch.object(glance.GlanceImageService, 'show')
+    def test_update_base_image_ref(self, show, translate_from_glance):
+        image_id = mock.sentinel.image_id
+        client = mock.Mock(call=mock.Mock())
+        service = glance.GlanceImageService(client=client)
+        data = '*' * 256
+        show.return_value = {}
+        translate_from_glance.return_value = {}
+
+        service.update(self.context, image_id, {}, data,
+                       base_image_ref=123)
+        calls = [mock.call.call(
+            self.context, 'upload', image_id, data, base_image_ref=123),
+            mock.call.call(self.context, 'get', image_id)]
+        client.assert_has_calls(calls, any_order=True)
+
+    def test_call_with_additional_headers(self):
+        glance_wrapper = glance.GlanceClientWrapper()
+        fake_client = mock.Mock()
+        self.mock_object(glance_wrapper, 'client', fake_client)
+        glance_wrapper.call(self.context, 'upload',
+                            {},
+                            store_id='xyz',
+                            base_image_ref=123)
+        self.assertDictEqual({
+            'x-image-meta-store': 'xyz',
+            'x-openstack-base-image-ref': 123},
+            fake_client.http_client.additional_headers)
 
     def test_delete(self):
         fixture1 = self._make_fixture(name='test image 1')
@@ -429,7 +597,6 @@ class TestGlanceImageService(test.TestCase):
         expected = {
             'id': image_id,
             'name': 'image1',
-            'is_public': True,
             'protected': None,
             'size': None,
             'min_disk': None,
@@ -439,19 +606,18 @@ class TestGlanceImageService(test.TestCase):
             'checksum': None,
             'created_at': self.NOW_DATETIME,
             'updated_at': self.NOW_DATETIME,
-            'deleted_at': None,
             'deleted': None,
             'status': None,
-            'properties': {},
+            'properties': {'is_public': True, 'properties': {}},
             'owner': None,
+            'visibility': None
         }
         self.assertEqual(expected, image_meta)
 
     def test_show_raises_when_no_authtoken_in_the_context(self):
         fixture = self._make_fixture(name='image1',
                                      is_public=False,
-                                     protected=False,
-                                     properties={'one': 'two'})
+                                     protected=False)
         image_id = self.service.create(self.context, fixture)['id']
         self.context.auth_token = False
         self.assertRaises(exception.ImageNotFound,
@@ -467,7 +633,6 @@ class TestGlanceImageService(test.TestCase):
             {
                 'id': image_id,
                 'name': 'image10',
-                'is_public': True,
                 'protected': None,
                 'size': None,
                 'min_disk': None,
@@ -477,11 +642,11 @@ class TestGlanceImageService(test.TestCase):
                 'checksum': None,
                 'created_at': self.NOW_DATETIME,
                 'updated_at': self.NOW_DATETIME,
-                'deleted_at': None,
                 'deleted': None,
                 'status': None,
-                'properties': {},
+                'properties': {'is_public': True, 'properties': {}},
                 'owner': None,
+                'visibility': None
             },
         ]
         self.assertEqual(expected, image_metas)
@@ -530,6 +695,20 @@ class TestGlanceImageService(test.TestCase):
         self.flags(glance_num_retries=1)
         service.download(self.context, image_id, writer)
 
+    def test_download_no_data(self):
+        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
+            """Returns None instead of an iterator."""
+            def data(self, image_id):
+                return None
+
+        client = MyGlanceStubClient()
+        service = self._create_image_service(client)
+        image_id = 'fake-image-uuid'
+        e = self.assertRaises(exception.ImageDownloadFailed, service.download,
+                              self.context, image_id)
+        self.assertIn('image contains no data', six.text_type(e))
+        self.assertIn(image_id, six.text_type(e))
+
     def test_client_forbidden_converts_to_imagenotauthed(self):
         class MyGlanceStubClient(glance_stubs.StubGlanceClient):
             """A client that raises a Forbidden exception."""
@@ -543,6 +722,62 @@ class TestGlanceImageService(test.TestCase):
         self.assertRaises(exception.ImageNotAuthorized, service.download,
                           self.context, image_id, writer)
 
+    def test_client_translated_exc_includes_original_traceback(self):
+        image_id = 1  # doesn't matter
+        original_exc = glanceclient.exc.Forbidden(image_id)
+
+        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
+            def get(self, image_id):
+                raise original_exc
+
+        client = MyGlanceStubClient()
+        service = self._create_image_service(client)
+        writer = NullWriter()
+
+        exc = self.assertRaises(exception.ImageNotAuthorized,
+                                service.download,
+                                self.context,
+                                image_id,
+                                writer)
+
+        original = traceback.extract_tb(original_exc.__traceback__)
+        original.reverse()
+        received = traceback.extract_tb(exc.__traceback__)
+        received.reverse()
+        # verify that we have the same traceback as original_exc
+        for orig, recd in zip(original, received):
+            self.assertEqual(orig, recd)
+
+        # note that the received exception contains more frames in
+        # its traceback than the original
+        self.assertGreater(len(received), len(original))
+
+    def test_plain_translated_exc_includes_original_traceback(self):
+        original_exc = glanceclient.exc.Forbidden()
+
+        class MyGlanceStubClient(glance_stubs.StubGlanceClient):
+            def list(self):
+                raise original_exc
+
+        client = MyGlanceStubClient()
+        service = self._create_image_service(client)
+
+        exc = self.assertRaises(exception.NotAuthorized,
+                                service.detail,
+                                self.context)
+
+        original = traceback.extract_tb(original_exc.__traceback__)
+        original.reverse()
+        received = traceback.extract_tb(exc.__traceback__)
+        received.reverse()
+        # verify that we have the same traceback as original_exc
+        for orig, recd in zip(original, received):
+            self.assertEqual(orig, recd)
+
+        # note that the received exception contains more frames in
+        # its traceback than the original
+        self.assertGreater(len(received), len(original))
+
     def test_client_httpforbidden_converts_to_imagenotauthed(self):
         class MyGlanceStubClient(glance_stubs.StubGlanceClient):
             """A client that raises a HTTPForbidden exception."""
@@ -593,9 +828,9 @@ class TestGlanceImageService(test.TestCase):
         image_id = self.service.create(self.context, fixture)['id']
         writer = NullWriter()
         self.flags(allowed_direct_url_schemes=['file'])
-        self.flags(glance_api_version=2)
         self.service.download(self.context, image_id, writer)
         mock_copyfileobj.assert_called_once_with(mock.ANY, writer)
+        mock_open.assert_called_once_with('/tmp/test', 'rb')
 
     @mock.patch('six.moves.builtins.open')
     @mock.patch('shutil.copyfileobj')
@@ -608,7 +843,6 @@ class TestGlanceImageService(test.TestCase):
         image_id = self.service.create(self.context, fixture)['id']
         writer = NullWriter()
         self.flags(allowed_direct_url_schemes=['file'])
-        self.flags(glance_api_version=2)
         self.service.download(self.context, image_id, writer)
         self.assertIsNone(mock_copyfileobj.call_args)
 
@@ -662,7 +896,6 @@ class TestGlanceImageService(test.TestCase):
         expected = {
             'id': 1,
             'name': None,
-            'is_public': None,
             'protected': None,
             'size': None,
             'min_disk': None,
@@ -677,13 +910,14 @@ class TestGlanceImageService(test.TestCase):
             'status': None,
             'properties': {},
             'owner': None,
+            'visibility': None,
+            'cinder_encryption_key_id': None
         }
         self.assertEqual(expected, actual)
 
     @mock.patch('cinder.image.glance.CONF')
     def test_v2_passes_visibility_param(self, config):
 
-        config.glance_api_version = 2
         config.glance_num_retries = 0
 
         metadata = {
@@ -723,7 +957,6 @@ class TestGlanceImageService(test.TestCase):
     @mock.patch('cinder.image.glance.CONF')
     def test_extracting_v2_boot_properties(self, config):
 
-        config.glance_api_version = 2
         config.glance_num_retries = 0
 
         metadata = {
@@ -765,7 +998,6 @@ class TestGlanceImageService(test.TestCase):
         self.assertEqual(expected, actual)
 
     def test_translate_to_glance(self):
-        self.flags(glance_api_version=1)
         client = glance_stubs.StubGlanceClient()
         service = self._create_image_service(client)
 
@@ -774,28 +1006,13 @@ class TestGlanceImageService(test.TestCase):
             'size': 2,
             'min_disk': 2,
             'min_ram': 2,
+            'cinder_encryption_key_deletion_policy': 'outer',
+            # note that a key duplicated in the 'properties' dict
+            # will overwrite the "outer" value
             'properties': {'kernel_id': 'foo',
                            'ramdisk_id': 'bar',
-                           'x_billinginfo': '123'},
-        }
-
-        actual = service._translate_to_glance(metadata)
-        expected = metadata
-        self.assertEqual(expected, actual)
-
-    def test_translate_to_glance_v2(self):
-        self.flags(glance_api_version=2)
-        client = glance_stubs.StubGlanceClient()
-        service = self._create_image_service(client)
-
-        metadata = {
-            'id': 1,
-            'size': 2,
-            'min_disk': 2,
-            'min_ram': 2,
-            'properties': {'kernel_id': 'foo',
-                           'ramdisk_id': 'bar',
-                           'x_billinginfo': '123'},
+                           'x_billinginfo': '123',
+                           'cinder_encryption_key_deletion_policy': 'NOPE'},
         }
 
         actual = service._translate_to_glance(metadata)
@@ -804,43 +1021,41 @@ class TestGlanceImageService(test.TestCase):
             'size': 2,
             'min_disk': 2,
             'min_ram': 2,
+            'cinder_encryption_key_deletion_policy': 'NOPE',
             'kernel_id': 'foo',
             'ramdisk_id': 'bar',
             'x_billinginfo': '123',
         }
         self.assertEqual(expected, actual)
 
+    def test_translate_to_glance_no_properties_element(self):
+        """Show _translate does not remove arbitrary flat properties"""
+        client = glance_stubs.StubGlanceClient()
+        service = self._create_image_service(client)
 
-class TestGlanceClientVersion(test.TestCase):
-    """Tests the version of the glance client generated."""
+        metadata = {
+            'id': 1,
+            'cinder_encryption_key_deletion_policy': 'baz',
+            'size': 2,
+            'min_disk': 2,
+            'min_ram': 2,
+            'kernel_id': 'foo',
+            'ramdisk_id': 'bar',
+            'x_billinginfo': '123',
+        }
 
-    @mock.patch('cinder.image.glance.glanceclient.Client')
-    def test_glance_version_by_flag(self, _mockglanceclient):
-        """Test glance version set by flag is honoured."""
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292)
-        self.assertEqual('1', _mockglanceclient.call_args[0][0])
-        self.flags(glance_api_version=2)
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292)
-        self.assertEqual('2', _mockglanceclient.call_args[0][0])
-        CONF.reset()
-
-    @mock.patch('cinder.image.glance.glanceclient.Client')
-    def test_glance_version_by_arg(self, _mockglanceclient):
-        """Test glance version set by arg to GlanceClientWrapper"""
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292, version=1)
-        self.assertEqual('1', _mockglanceclient.call_args[0][0])
-        glance.GlanceClientWrapper('fake', 'fake_host', 9292, version=2)
-        self.assertEqual('2', _mockglanceclient.call_args[0][0])
-
-    @mock.patch('cinder.image.glance.glanceclient.Client')
-    @mock.patch('cinder.image.glance.get_api_servers',
-                return_value=itertools.cycle([(False, 'localhost:9292')]))
-    def test_call_glance_version_by_arg(self, api_servers, _mockglanceclient):
-        """Test glance version set by arg to GlanceClientWrapper"""
-        glance_wrapper = glance.GlanceClientWrapper()
-        glance_wrapper.call('fake_context', 'method', version=2)
-
-        self.assertEqual('2', _mockglanceclient.call_args[0][0])
+        actual = service._translate_to_glance(metadata)
+        expected = {
+            'id': 1,
+            'cinder_encryption_key_deletion_policy': 'baz',
+            'size': 2,
+            'min_disk': 2,
+            'min_ram': 2,
+            'kernel_id': 'foo',
+            'ramdisk_id': 'bar',
+            'x_billinginfo': '123',
+        }
+        self.assertEqual(expected, actual)
 
     @mock.patch('cinder.image.glance.glanceclient.Client')
     @mock.patch('cinder.image.glance.get_api_servers',
@@ -853,8 +1068,7 @@ class TestGlanceClientVersion(test.TestCase):
             side_effect=glanceclient.exc.HTTPOverLimit)
         self.mock_object(glance_wrapper, 'client', fake_client)
         self.assertRaises(exception.ImageLimitExceeded,
-                          glance_wrapper.call, 'fake_context', 'method',
-                          version=2)
+                          glance_wrapper.call, 'fake_context', 'method')
 
 
 def _create_failing_glance_client(info):
@@ -874,50 +1088,113 @@ class TestGlanceImageServiceClient(test.TestCase):
     def setUp(self):
         super(TestGlanceImageServiceClient, self).setUp()
         self.context = context.RequestContext('fake', 'fake', auth_token=True)
-        self.mock_object(glance.time, 'sleep', mock.Mock(return_value=None))
+        self.mock_object(glance.time, 'sleep', return_value=None)
+        service_auth.reset_globals()
 
-    def test_create_glance_client(self):
-        self.flags(auth_strategy='keystone')
-        self.flags(glance_request_timeout=60)
-
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
-                self.assertEqual('1', version)
-                self.assertEqual("http://fake_host:9292", args[0])
-                self.assertTrue(kwargs['token'])
-                self.assertEqual(60, kwargs['timeout'])
-
-        self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
-        client = glance._create_glance_client(self.context, 'fake_host:9292',
-                                              False)
-        self.assertIsInstance(client, MyGlanceStubClient)
-
-    def test_create_glance_client_auth_strategy_is_not_keystone(self):
-        self.flags(auth_strategy='noauth')
-        self.flags(glance_request_timeout=60)
-
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
-                self.assertEqual('1', version)
-                self.assertEqual('http://fake_host:9292', args[0])
-                self.assertNotIn('token', kwargs)
-                self.assertEqual(60, kwargs['timeout'])
-
-        self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
-        client = glance._create_glance_client(self.context, 'fake_host:9292',
-                                              False)
-        self.assertIsInstance(client, MyGlanceStubClient)
-
-    def test_create_glance_client_glance_request_default_timeout(self):
+    @mock.patch('cinder.service_auth.get_auth_plugin')
+    @mock.patch.object(ks_session.Session, 'load_from_options')
+    def test_create_glance_client_with_protocol_http(
+            self, mock_load, mock_get_auth_plugin):
+        glance._SESSION = None
         self.flags(auth_strategy='keystone')
         self.flags(glance_request_timeout=None)
 
         class MyGlanceStubClient(object):
             def __init__(inst, version, *args, **kwargs):
-                self.assertEqual("1", version)
+                self.assertEqual('2', version)
                 self.assertEqual("http://fake_host:9292", args[0])
-                self.assertTrue(kwargs['token'])
                 self.assertNotIn('timeout', kwargs)
+                self.assertIn("session", kwargs)
+                self.assertIn("auth", kwargs)
+
+        config_options = {'insecure': False,
+                          'cacert': None,
+                          'key': None,
+                          'cert': None,
+                          'timeout': None,
+                          'split_loggers': False}
+
+        mock_get_auth_plugin.return_value = context._ContextAuthPlugin
+        mock_load.return_value = session.Session
+        self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              False)
+        self.assertIsInstance(client, MyGlanceStubClient)
+        mock_get_auth_plugin.assert_called_once_with(self.context)
+        mock_load.assert_called_once_with(**config_options)
+
+    @mock.patch('cinder.service_auth.get_auth_plugin')
+    @mock.patch.object(ks_session.Session, 'load_from_options')
+    def test_create_glance_client_with_protocol_https(
+            self, mock_load, mock_get_auth_plugin):
+        glance._SESSION = None
+        self.flags(auth_strategy='keystone')
+        self.flags(glance_request_timeout=60)
+        self.flags(
+            glance_ca_certificates_file='/opt/stack/data/ca-bundle.pem')
+        self.flags(glance_certfile='/opt/stack/data/cert.pem')
+        self.flags(glance_keyfile='/opt/stack/data/key.pem')
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual('2', version)
+                self.assertEqual("https://fake_host:9292", args[0])
+                self.assertNotIn('timeout', kwargs)
+                self.assertIn("session", kwargs)
+                self.assertIn("auth", kwargs)
+
+        config_options = {'insecure': False,
+                          'cacert': '/opt/stack/data/ca-bundle.pem',
+                          'cert': '/opt/stack/data/cert.pem',
+                          'key': '/opt/stack/data/key.pem',
+                          'timeout': 60,
+                          'split_loggers': False}
+
+        mock_get_auth_plugin.return_value = context._ContextAuthPlugin
+        mock_load.return_value = session.Session
+        self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              True)
+        self.assertIsInstance(client, MyGlanceStubClient)
+        mock_get_auth_plugin.assert_called_once_with(self.context)
+        mock_load.assert_called_once_with(**config_options)
+
+    def test_create_glance_client_auth_strategy_noauth_with_protocol_https(
+            self):
+        self.flags(auth_strategy='noauth')
+        self.flags(glance_request_timeout=60)
+        self.flags(glance_api_insecure=False)
+        self.flags(
+            glance_ca_certificates_file='/opt/stack/data/ca-bundle.pem')
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual('2', version)
+                self.assertEqual('https://fake_host:9292', args[0])
+                self.assertEqual(60, kwargs['timeout'])
+                self.assertNotIn("session", kwargs)
+                self.assertNotIn("auth", kwargs)
+                self.assertEqual(
+                    '/opt/stack/data/ca-bundle.pem', kwargs['cacert'])
+                self.assertEqual(False, kwargs['insecure'])
+
+        self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
+        client = glance._create_glance_client(self.context, 'fake_host:9292',
+                                              True)
+        self.assertIsInstance(client, MyGlanceStubClient)
+
+    def test_create_glance_client_auth_strategy_noauth_with_protocol_http(
+            self):
+        self.flags(auth_strategy='noauth')
+        self.flags(glance_request_timeout=None)
+
+        class MyGlanceStubClient(object):
+            def __init__(inst, version, *args, **kwargs):
+                self.assertEqual('2', version)
+                self.assertEqual("http://fake_host:9292", args[0])
+                self.assertNotIn('timeout', kwargs)
+                self.assertNotIn("session", kwargs)
+                self.assertNotIn("auth", kwargs)
 
         self.mock_object(glance.glanceclient, 'Client', MyGlanceStubClient)
         client = glance._create_glance_client(self.context, 'fake_host:9292',
