@@ -15,10 +15,10 @@
 
 from copy import deepcopy
 import datetime
-import hashlib
 import re
 
 from oslo_log import log as logging
+from oslo_utils.secretutils import md5
 from oslo_utils import strutils
 from oslo_utils import units
 import packaging.version
@@ -132,7 +132,6 @@ POWERMAX_ARRAY = 'powermax_array'
 POWERMAX_SRP = 'powermax_srp'
 POWERMAX_SERVICE_LEVEL = 'powermax_service_level'
 POWERMAX_PORT_GROUPS = 'powermax_port_groups'
-POWERMAX_SNAPVX_UNLINK_LIMIT = 'powermax_snapvx_unlink_limit'
 POWERMAX_ARRAY_TAG_LIST = 'powermax_array_tag_list'
 POWERMAX_SHORT_HOST_NAME_TEMPLATE = 'powermax_short_host_name_template'
 POWERMAX_PORT_GROUP_NAME_TEMPLATE = 'powermax_port_group_name_template'
@@ -213,6 +212,10 @@ PORT_ID = 'portId'
 # Revert snapshot exception
 REVERT_SS_EXC = 'Link must be fully copied for this operation to proceed'
 
+# extra specs
+IS_TRUE = ['<is> True', 'True', 'true', True]
+IS_FALSE = ['<is> False', 'False', 'false', False]
+
 
 class PowerMaxUtils(object):
     """Utility class for Rest based PowerMax volume drivers.
@@ -431,7 +434,7 @@ class PowerMaxUtils(object):
         :returns: uuid
         """
         input_str = input_str.lower()
-        m = hashlib.md5()
+        m = md5(usedforsecurity=False)
         m.update(input_str.encode('utf-8'))
         return m.hexdigest()
 
@@ -505,7 +508,7 @@ class PowerMaxUtils(object):
         compression_disabled = False
 
         if extra_specs.get(DISABLECOMPRESSION, False):
-            if strutils.bool_from_string(extra_specs.get(DISABLECOMPRESSION)):
+            if extra_specs.get(DISABLECOMPRESSION) in IS_TRUE:
                 compression_disabled = True
         else:
             if extra_specs.get(SLO):
@@ -1054,6 +1057,9 @@ class PowerMaxUtils(object):
         if vol_head['userDefinedIdentifier'][0:3] == 'OS-':
             return False
 
+        if vol_head.get('numStorageGroups', 0) > 1:
+            return False
+
         return True
 
     @staticmethod
@@ -1781,6 +1787,9 @@ class PowerMaxUtils(object):
         :param storage_group_name: the storage group name -- str
         :returns: updated payload -- dict
         """
+        create_new_vol = {"create_new_volumes": "True"}
+        payload["editStorageGroupActionParam"]["expandStorageGroupParam"][
+            "addVolumeParam"].update(create_new_vol)
         remote_dict = {"remoteSymmSGInfoParam": {
             "remote_symmetrix_1_id": remote_array_id,
             "remote_symmetrix_1_sgs": [storage_group_name],
@@ -2054,6 +2063,7 @@ class PowerMaxUtils(object):
         :returns: array_id, srp, service_level, workload -- str, str, str, str
         """
         array_id, srp, service_level, workload = str(), str(), str(), str()
+
         pool_details = pool_name.split('+')
         if len(pool_details) == 4:
             array_id = pool_details[3]
@@ -2079,3 +2089,12 @@ class PowerMaxUtils(object):
             service_level = str()
 
         return array_id, srp, service_level, workload
+
+    @staticmethod
+    def convert_to_string(in_value):
+        """Convert to string if value is an int
+
+        :param in_value: the input (most likely a str or int)
+        :returns: str
+        """
+        return in_value if isinstance(in_value, str) else str(in_value)
